package de.bioforscher.singa.core.parameters;

import org.junit.Test;

import java.util.Arrays;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class ParameterSamplingTests {

    @Test
    public void testMixedParameterRetrieval() {
        DoubleParameter doubleParameter = new DoubleParameter("A", 0.0, 10.0);
        ParameterValue<Double> doubleParameterValue = new ParameterValue<>(doubleParameter, 3.0);
        BooleanParameter booleanParameter = new BooleanParameter("B");
        ParameterValue<Boolean> booleanParameterValue = new ParameterValue<>(booleanParameter, true);

        MixedParameterList pList = new MixedParameterList();
        pList.add(doubleParameterValue);
        pList.add(booleanParameterValue);

        ParameterValue<Double> retrievedDoubleParameterValue = pList.getValue(0, Double.class);
        ParameterValue<Boolean> retrievedBooleanParameterValue = pList.getValue(1, Boolean.class);

        assertEquals(retrievedDoubleParameterValue.toString(), doubleParameterValue.toString());
        assertEquals(retrievedBooleanParameterValue.toString(), booleanParameterValue.toString());
    }

    @Test
    public void testDoubleParameterValueSampling() {
        DoubleParameter parameter = new DoubleParameter("A", 0.0, 10.0);
        UniqueParameterList<Double> actual = ParameterSampler.sample(parameter, 11);
        List<Double> expected = Arrays.asList(0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0);
        for (int i = 0; i < expected.size(); i++) {
            assertEquals(expected.get(i), actual.getValues().get(i).getValue(), 0.0);
        }
    }

    @Test
    public void testIntegerParameterValueSampling() {
        IntegerParameter parameter = new IntegerParameter("B", 0, 10);
        UniqueParameterList<Integer> actual = ParameterSampler.sample(parameter, 11);
        List<Integer> expected = Arrays.asList(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10);
        for (int i = 0; i < expected.size(); i++) {
            assertEquals(expected.get(i), actual.getValues().get(i).getValue(), 0.0);
        }
    }

    @Test
    public void testBooleanParameterValueSampling() {
        BooleanParameter parameter = new BooleanParameter("C");
        UniqueParameterList<Boolean> actual = ParameterSampler.sample(parameter);
        List<Boolean> expected = Arrays.asList(true, false);
        assertTrue(actual.getValues().get(0).getValue() == expected.get(0));
        assertTrue(actual.getValues().get(1).getValue() == expected.get(1));
    }

}
