package de.bioforscher.singa.javafx.renderer.molecules;

import de.bioforscher.singa.javafx.renderer.graphs.GraphRenderOptions;
import de.bioforscher.singa.javafx.renderer.graphs.GraphRenderer;
import de.bioforscher.singa.mathematics.geometry.edges.LineSegment;
import de.bioforscher.singa.structure.model.molecules.MoleculeAtom;
import de.bioforscher.singa.structure.model.molecules.MoleculeBond;
import de.bioforscher.singa.structure.model.molecules.MoleculeGraph;

public class MoleculeGraphRenderer extends GraphRenderer<MoleculeAtom, MoleculeBond, Integer, MoleculeGraph> {

    private final MoleculeRendererOptions moleculeOptions = new MoleculeRendererOptions();

    public MoleculeGraphRenderer() {
        GraphRenderOptions options = new GraphRenderOptions();
        options.setNodeDiameter(22);
        setRenderingOptions(options);
    }

    @Override
    protected void drawNode(MoleculeAtom atom) {
        // draw node
        getGraphicsContext().setFill(moleculeOptions.getColorForElement(atom.getElement().getSymbol()));
        drawPoint(atom.getPosition(), getRenderingOptions().getNodeDiameter());
        // draw outline
        getGraphicsContext().setStroke(getRenderingOptions().getEdgeColor());
        circlePoint(atom.getPosition(), getRenderingOptions().getNodeDiameter());
        // draw text
        getGraphicsContext().setFill(getRenderingOptions().getIdentifierTextColor());
        drawTextCenteredOnPoint(atom.getElement().toString() + "." + atom.getIdentifier(), atom.getPosition());
    }

    @Override
    protected void drawEdge(MoleculeBond bond) {
        // set color and width
        getGraphicsContext().setLineWidth(getRenderingOptions().getEdgeThickness());
        getGraphicsContext().setStroke(getRenderingOptions().getEdgeColor());
        // draw bond
        drawBond(bond);
    }


    private void drawBond(MoleculeBond bond) {
        LineSegment connectingSegment = new LineSegment(bond.getSource().getPosition(), bond.getTarget().getPosition());
        switch (bond.getType()) {
            case DOUBLE_BOND: {
                // draw upper parallel
                LineSegment upperParallelSegment = connectingSegment.getParallelSegment((getRenderingOptions().getNodeDiameter() / 2.0) * 0.5);
                drawLineSegment(upperParallelSegment);
                // draw lower parallel
                LineSegment lowerParallelSegment = connectingSegment.getParallelSegment((-getRenderingOptions().getNodeDiameter() / 2.0) * 0.5);
                drawLineSegment(lowerParallelSegment);
                break;
            }
            case AROMATIC_BOND: {
                // draw upper parallel
                LineSegment upperParallelSegment = connectingSegment.getParallelSegment((getRenderingOptions().getNodeDiameter() / 2.0) * 0.5);
                drawLineSegment(upperParallelSegment);
                // draw lower parallel
                LineSegment lowerParallelSegment = connectingSegment.getParallelSegment((-getRenderingOptions().getNodeDiameter() / 2.0) * 0.5);
                dashLineSegment(lowerParallelSegment, 2d, 4d);
                break;
            }
            default:
                // draw single bond
                drawLineSegment(connectingSegment);
                break;
        }


    }

}
